@@ -11,10 +11,12 @@ package org.gnu.gnome;
 
 import java.util.Vector;
 
+import org.gnu.glib.EventMap;
 import org.gnu.glib.Type;
 import org.gnu.gnome.event.ClientEvent;
 import org.gnu.gnome.event.ClientListener;
 import org.gnu.gtk.GtkObject;
+import org.gnu.gtk.event.GtkEventType;
 /**
  * The GnomeClient object makes it possible for your application to
  * save session information when the user logs out. If, when logging
@@ -35,7 +37,6 @@ public class Client extends GtkObject {
 	 */
 	public Client() {
 		super(gnome_client_new());
-		initializeEventHandlers();
 	}
 	
 	/**
@@ -43,7 +44,6 @@ public class Client extends GtkObject {
 	 */
 	protected Client(int handle) {
 		super(handle);
-		initializeEventHandlers();
 	}
 
 	/**
@@ -182,6 +182,12 @@ public class Client extends GtkObject {
 		int i = findListener(clientListeners, listener);
 		if (i == -1) {
 			if (null == clientListeners) {
+				evtMap.initialize(this, ClientEvent.Type.CONNECT);
+				evtMap.initialize(this, ClientEvent.Type.DIE);
+				evtMap.initialize(this, ClientEvent.Type.DISCONNECT);
+				evtMap.initialize(this, ClientEvent.Type.SAVE_COMPLETE);
+				evtMap.initialize(this, ClientEvent.Type.SAVE_YOURSELF);
+				evtMap.initialize(this, ClientEvent.Type.SHUTDOWN_CANCELLED);
 				clientListeners = new Vector();
 			}
 			clientListeners.addElement(listener);
@@ -197,6 +203,12 @@ public class Client extends GtkObject {
 			clientListeners.remove(i);
 		}
 		if (0 == clientListeners.size()) {
+			evtMap.uninitialize(this, ClientEvent.Type.CONNECT);
+			evtMap.uninitialize(this, ClientEvent.Type.DIE);
+			evtMap.uninitialize(this, ClientEvent.Type.DISCONNECT);
+			evtMap.uninitialize(this, ClientEvent.Type.SAVE_COMPLETE);
+			evtMap.uninitialize(this, ClientEvent.Type.SAVE_YOURSELF);
+			evtMap.uninitialize(this, ClientEvent.Type.SHUTDOWN_CANCELLED);
 			clientListeners = null;
 		}
 	}
@@ -251,14 +263,36 @@ public class Client extends GtkObject {
 		fireClientEvent(new ClientEvent(this, ClientEvent.Type.SHUTDOWN_CANCELLED));
 	}
 
-	protected void initializeEventHandlers() {
-		addEventHandler("connect", "handleConnect", this);
-		addEventHandler("die", "handleDie", this);
-		addEventHandler("disconnect", "handleDisconnect", this);
-		addEventHandler("save_complete", "handleSaveComplete", this);
-		addEventHandler("save_yourself", "handleSaveYourself", this);
-		addEventHandler("shutdown_cancelled", "handleShutdownCancelled", this);
+	public Class getEventListenerClass(String signal) {
+		Class cls = evtMap.getEventListenerClass(signal);
+		if (cls == null) cls = super.getEventListenerClass(signal);
+		return cls;
 	}
+
+	public GtkEventType getEventType(String signal) {
+		GtkEventType et = evtMap.getEventType(signal);
+		if (et == null) et = super.getEventType(signal);
+		return et;
+	}
+
+	private static EventMap evtMap = new EventMap();
+	static {
+		addEvents(evtMap);
+	}
+
+	/**
+	* Implementation method to build an EventMap for this widget class.
+	* Not useful (or supported) for application use.
+	*/
+	private static void addEvents(EventMap anEvtMap) {
+		anEvtMap.addEvent("connect", "handleConnect", ClientEvent.Type.CONNECT, ClientListener.class);
+		anEvtMap.addEvent("die", "handleDie", ClientEvent.Type.DIE, ClientListener.class);
+		anEvtMap.addEvent("disconnect", "handleDisconnect", ClientEvent.Type.DISCONNECT, ClientListener.class);
+		anEvtMap.addEvent("save_complete", "handleSaveComplete", ClientEvent.Type.SAVE_COMPLETE, ClientListener.class);
+		anEvtMap.addEvent("save_yourself", "handleSaveYourself", ClientEvent.Type.SAVE_YOURSELF, ClientListener.class);
+		anEvtMap.addEvent("shutdown_cancelled", "handleShutdownCancelled", ClientEvent.Type.SHUTDOWN_CANCELLED, ClientListener.class);
+	}
+
 	/****************************************
 	 * BEGINNING OF GENERATED CODE
 	 ****************************************/
