@@ -36,21 +36,14 @@ public class ColorPicker extends Button {
 	 * Creates a new instance of ColorPicker
 	 */
 	public ColorPicker() {
-		noopInit();
+		super(gnome_color_picker_new());
 	}
 
 	/**
 	 * Construct a ColorPicker using a handle to a native resource.
 	 */
 	public ColorPicker(int handle) {
-		setHandle(handle);
-	}
-
-	protected void noopInit() {
-		if (-1 == getHandle()) {
-			setHandle(gnome_color_picker_new());
-			initializeEventHandlers();
-		}
+		super(handle);
 	}
 
 	/**
@@ -648,6 +641,7 @@ public class ColorPicker extends Button {
 		int i = findListener(colorPickerListeners, listener);
 		if (i == -1) {
 			if (null == colorPickerListeners) {
+				evtMap.initialize(this, ColorPickerEvent.Type.COLOR_SET);
 				colorPickerListeners = new Vector();
 			}
 			colorPickerListeners.addElement(listener);
@@ -663,6 +657,7 @@ public class ColorPicker extends Button {
 			colorPickerListeners.remove(i);
 		}
 		if (0 == colorPickerListeners.size()) {
+			evtMap.uninitialize(this, ColorPickerEvent.Type.COLOR_SET);
 			colorPickerListeners = null;
 		}
 	}
@@ -684,17 +679,16 @@ public class ColorPicker extends Button {
 		fireColorPickerEvent(new ColorPickerEvent(this));
 	}
 
-	protected void initializeEventHandlers() {
-		super.initializeEventHandlers();
-		evtMap.initialize(this);
-	}
-
 	public Class getEventListenerClass(String signal) {
-		return evtMap.getListenerClass(signal);
+		Class cls = evtMap.getEventListenerClass(signal);
+		if (cls == null) cls = super.getEventListenerClass(signal);
+		return cls;
 	}
 
 	public GtkEventType getEventType(String signal) {
-		return evtMap.getEventType(signal);
+		GtkEventType et = evtMap.getEventType(signal);
+		if (et == null) et = super.getEventType(signal);
+		return et;
 	}
 
 	private static EventMap evtMap = new EventMap();
